# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2019. All Rights Reserved.
# pragma pylint: disable=unused-argument, no-self-use

import requests
from resilient_lib.components.integration_errors import IntegrationError


class RequestsCommon:
    """
    This class represents common functions around the use of the requests package for REST based APIs.
    It incorporates the app.config section "integrations" which can be used to define a common set of proxies
    for use by all functions using this library:

    [integrations]
    http_proxy=
    https_proxy=

    Similar properties may exist in the function's section which would override the [integrations] properties.
    """
    def __init__(self, opts=None, function_opts=None):
        # capture the properties for the integration as well as the global settings for all integrations for proxy urls
        self.integration_options = opts.get('integrations', None) if opts else None
        self.function_opts = function_opts

    def get_proxies(self):
        """ proxies can be specified globally for all integrations or specifically per function """
        proxies = None
        if self.integration_options and (self.integration_options.get("http_proxy") or self.integration_options.get("https_proxy")):
            proxies = {'http': self.integration_options.get("http_proxy"), 'https': self.integration_options.get("https_proxy")}

        if self.function_opts and (self.function_opts.get("http_proxy") or self.function_opts.get("https_proxy")):
            proxies = {'http': self.function_opts.get("http_proxy"), 'https': self.function_opts.get("https_proxy")}

        return proxies

    def execute_call(self, verb, url, payload={}, log=None, basicauth=None, verify_flag=True, headers=None,
                     proxies=None, timeout=None, resp_type='json', callback=None):
        """
        Function: perform the http API call. Different types of http operations are supported:
        GET, HEAD, PATCH, POST, PUT, DELETE
        Errors raise IntegrationError
        If a callback method is provided, then it's called to handle the error

        When using argument 'json' Content-Type will automatically be set to "application/json" by requests lib.

        :param verb: GET, HEAD, PATCH, POST, PUT, DELETE
        :param url:
        :param basicauth: used for basic authentication - (user, password)
        :param payload:
        :param log: optional log statement
        :param verify_flag: True/False - False used for debugging generally
        :param headers: dictionary of http headers
        :param proxies: http and https proxies for call
        :param timeout: timeout before call should abort
        :param resp_type: type of output to return: json, text, bytes
        :param callback: callback routine used to handle errors
        :return: json of returned data
        """

        try:
            (payload and log) and log.debug(payload)

            if verb.lower() not in ('get', 'post', 'put', 'patch', 'delete', 'head'):
                raise IntegrationError("unknown verb {}".format(verb))

            if proxies is None:
                proxies = self.get_proxies()

            if verb.lower() == 'post':

                content_type = get_case_insensitive_key_value(headers, "Content-Type")

                if is_payload_in_json(content_type):
                    resp = requests.request(verb.upper(), url, verify=verify_flag, headers=headers, json=payload,
                                            auth=basicauth, timeout=timeout, proxies=proxies)
                else:
                    resp = requests.request(verb.upper(), url, verify=verify_flag, headers=headers, data=payload,
                                            auth=basicauth, timeout=timeout, proxies=proxies)
            else:
                resp = requests.request(verb.upper(), url, verify=verify_flag, headers=headers, params=payload,
                                        auth=basicauth, timeout=timeout, proxies=proxies)

            if resp is None:
                raise IntegrationError('no response returned')

            # custom handler for response handling?
            if callback:
                return callback(resp)

            # standard error handling
            if resp.status_code >= 300:
                # get the result
                # log resp.status_code in case resp.text isn't available
                raise IntegrationError(
                    "status_code: {}, msg: {}".format(resp.status_code, resp.text if resp.text else "N/A"))

            # check if anything returned
            log and log.debug(resp.text)

            # get the result
            if resp_type == 'json':
                r = resp.json()
            elif resp_type == 'text':
                r = resp.text
            elif resp_type == 'bytes':
                r = resp.content
            else:
                raise IntegrationError("incorrect response type: {}".format(resp_type))

            # Produce a IntegrationError with the return value
            return r      # json object needed, not a string representation
        except Exception as err:
            msg = str(err)
            log and log.error(msg)
            raise IntegrationError(msg)


def is_payload_in_json(content_type):
    """
    Verify the content_type.
    If "Content-Type" is NOT specified pass the payload to the "json" argument - return True.

    If "Content-Type" is specified:
        - if the value is "application/json" pass the payload to the "json" argument - return True.
        - if the value is NOT "application/json" pass the payload to the "data" argument - return False.
    :param content_type:
    :return: True or False
    """

    if not content_type:
        return True

    return "application/json" in content_type.lower()


def get_case_insensitive_key_value(dictionary, key):
    """
    Get case insensitive key value from dictionary.
    :param dictionary:
    :param key:
    :return: value or None
    """
    if dictionary is None:
        return None

    return next((v for k, v in dictionary.items() if k.lower() == key.lower()), None)



# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2018. All Rights Reserved.
# pragma pylint: disable=unused-argument, no-self-use

import datetime
from bs4 import BeautifulSoup
from six import string_types
try:
    from HTMLParser import HTMLParser as htmlparser
except:
    from html.parser import HTMLParser as htmlparser


INCIDENT_FRAGMENT = '#incidents'
PAYLOAD_VERSION = "1.0"

def build_incident_url(url, incidentId):
    """
    build the url to link to an resilient incident
    :param url: base url
    :param incidentId:
    :return: full url
    """
    return '/'.join([url, INCIDENT_FRAGMENT, str(incidentId)])

def build_resilient_url(host, port):
    """
    build basic url to resilient instance
    :param host: host name
    :param port: port
    :return: base url
    """
    if host.lower().startswith("http"):
        return "{0}:{1}".format(host, port)

    return "https://{0}:{1}".format(host, port)

def clean_html(html_fragment):
    """
    Resilient textarea fields return html fragments. This routine will remove the html and insert any code within <div></div>
    with a linefeed
    :param html_fragment: str presenting the html to clean up
    :return: cleaned up code. This may not format well as no presentation of line feeds are preserved in the way supported by
       tags such as <br> or <ol>, <ul>, etc. See html2markdown for a better way to translate html input to markdown.
    """

    if not html_fragment or not isinstance(html_fragment, string_types):
        return html_fragment

    s = BeautifulSoup(unescape(html_fragment), "html.parser")

    return ' '.join(s.strings)

def unescape(data):
    """ Return unescaped data such as &gt; -> >, &quot -> ', etc.
    :param data: text to convert
    """
    if data is None:
        return None

    h = htmlparser()
    return h.unescape(data)


def validate_fields(field_list, kwargs):
    """
    ensure required fields are present. Throw ValueError if not
    :param field_list:
    :param kwargs:
    :return: no return
    """
    for field in field_list:
        if field not in kwargs or kwargs.get(field) == '':
            raise ValueError('Required field is missing or empty: '+field)


def get_file_attachment(res_client, incident_id, artifact_id=None, task_id=None, attachment_id=None):
    """
    call the Resilient REST API to get the attachment or artifact data
    :param res_client: required for communication back to resilient
    :param incident_id: required
    :param artifact_id: optional
    :param task_id: optional
    :param attachment_id: optional
    :return: byte string of attachment
    """

    if incident_id and artifact_id:
        data_uri = "/incidents/{}/artifacts/{}/contents".format(incident_id, artifact_id)
    elif attachment_id:
        if task_id:
            data_uri = "/tasks/{}/attachments/{}/contents".format(task_id, attachment_id)
        elif incident_id:
            data_uri = "/incidents/{}/attachments/{}/contents".format(incident_id, attachment_id)
        else:
            raise ValueError("task_id or incident_id must be specified with attachment")
    else:
        raise ValueError("artifact or attachment or incident id must be specified")

    # Get the data
    return res_client.get_content(data_uri)


def get_file_attachment_name(res_client, incident_id, artifact_id=None, task_id=None, attachment_id=None):
    """
    call the Resilient REST API to get the attachment or artifact attachment name
    :param res_client: required for communication back to resilient
    :param incident_id: required
    :param artifact_id: optional
    :param task_id: optional
    :param attachment_id: optional
    :return: file attachment name
    """

    name = ""
    if incident_id and artifact_id:
        name_url = "/incidents/{}/artifacts/{}".format(incident_id, artifact_id)
        name = str(res_client.get(name_url)["attachment"]["name"])
    elif attachment_id:
        if task_id:
            name_url = "/tasks/{}/attachments/{}".format(task_id, attachment_id)
            name = str(res_client.get(name_url)["name"])
        elif incident_id:
            name_url = "/incidents/{}/attachments/{}".format(incident_id, attachment_id)
            name = str(res_client.get(name_url)["name"])
        else:
            raise ValueError("task_id or incident_id must be specified with attachment")
    else:
        raise ValueError("artifact or attachment or incident id must be specified")

    # Return name string
    return name


def readable_datetime(timestamp, milliseconds=True, rtn_format='%Y-%m-%dT%H:%M:%SZ'):
    """
    convert an epoch timestamp to a string using a format
    :param timestamp:
    :param milliseconds: True = epoch in
    :param rtn_format: format of resulant string
    :return: string representation of timestamp
    """
    if milliseconds:
        ts = int(timestamp/1000)
    else:
        ts = timestamp

    return datetime.datetime.utcfromtimestamp(ts).strftime(rtn_format)

def str_to_bool(value):
    """Represents value as boolean.
    :param value:
    :rtype: bool
    """
    value = str(value).lower()
    return value in ('1', 'true', 'yes')



import copy
import json
import logging
import unittest
import pytest
from parameterized import parameterized
from resilient_lib.components.requests_common import RequestsCommon, get_case_insensitive_key_value, is_payload_in_json
from resilient_lib.components.integration_errors import IntegrationError

class TestFunctionRequests(unittest.TestCase):
    """ Tests for the attachment_hash function

    live tests against endpoints are listed below
    """

    URL_TEST_DATA_RESULTS = "https://api.ipify.org"
    URL_TEST_HTTP_VERBS = "https://postman-echo.com"
    URL_TEST_HTTP_STATUS_CODES = "http://httpstat.us"
    URL_TEST_PROXY = "https://gimmeproxy.com/api/getProxy"

    #rc.execute_call(verb, url, payload, log=None, basicauth=None, verify_flag=True, headers=None,
    #            proxies=None, timeout=None, resp_type=json, callback=None):
    LOG = logging.getLogger(__name__)

    def test_resilient_common_proxies(self):
        rc = RequestsCommon()
        self.assertIsNone(rc.get_proxies())

        integrations = { }

        rc = RequestsCommon(opts=integrations)
        self.assertIsNone(rc.get_proxies())

        integrations = { "integrations": { } }
        rc = RequestsCommon(integrations, None)
        self.assertIsNone(rc.get_proxies())

        integrations = { "integrations": { "https_proxy": "abc" } }
        rc = RequestsCommon(function_opts=None, opts=integrations)
        proxies = rc.get_proxies()
        self.assertEqual("abc", proxies['https'])
        self.assertIsNone(proxies['http'])

        integrations = { "integrations": { "https_proxy": "abc", 'http_proxy': 'def' } }
        rc = RequestsCommon(integrations)
        proxies = rc.get_proxies()
        self.assertEqual("abc", proxies['https'])
        self.assertEqual("def", proxies['http'])
        

    def test_resp_types(self):
        IPIFY = TestFunctionRequests.URL_TEST_DATA_RESULTS

        rc = RequestsCommon(None, None)

        # J S O N
        json_result = rc.execute_call("get", "{}?format=json".format(IPIFY), None)

        try:
            json.dumps(json_result)
        except (TypeError, OverflowError):
            self.fail("result is not json")

        # T E X T
        text_result = rc.execute_call("get", "{}?format=text".format(IPIFY), None, resp_type='text')
        self.assertIsNotNone(text_result)

        # B Y T E S
        bytes_result = rc.execute_call("get", "{}?format=text".format(IPIFY), None, resp_type='bytes')
        self.assertIsNotNone(bytes_result)
        self.assertTrue(isinstance(bytes_result, bytes))


    def test_verbs(self):
        URL = TestFunctionRequests.URL_TEST_HTTP_VERBS

        headers = {
            "Content-type": "application/json; charset=UTF-8"
        }

        payload = {
            'title': 'foo',
            'body': 'bar',
            'userId': 1
        }

        rc = RequestsCommon(None, None)


        # P O S T
        # test json argument without headers
        resp = rc.execute_call("post", "/".join((URL, "post")), payload, log=TestFunctionRequests.LOG)
        print (resp)
        self.assertEqual(resp['json'].get("body"), "bar")

        # test json argument with headers
        resp = rc.execute_call("post", "/".join((URL, "post")), payload, headers=headers, log=TestFunctionRequests.LOG)
        print (resp)
        self.assertEqual(resp['json'].get("body"), "bar")

        # test data argument
        headers_data = {
            "Content-type": "application/x-www-form-urlencoded"
        }
        resp = rc.execute_call("post", "/".join((URL, "post")), payload, headers=headers_data, log=TestFunctionRequests.LOG)
        print (resp)
        self.assertEqual(resp['json'].get("body"), "bar")


        # G E T
        resp = rc.execute_call("get", "/".join((URL, "get")), payload, log=TestFunctionRequests.LOG)
        self.assertTrue(resp['args'].get("userId"))
        self.assertEqual(resp['args'].get("userId"), '1')


        # P U T
        resp = rc.execute_call("put", "/".join((URL, "put")), payload, headers=headers, log=TestFunctionRequests.LOG)
        TestFunctionRequests.LOG.info(resp)
        self.assertTrue(resp['args'].get("title"))
        self.assertEqual(resp['args'].get("title"), 'foo')


        # P A T C H
        patch = {
            'title': 'patch'
        }

        resp = rc.execute_call("patch", "/".join((URL, "patch")), patch, headers=headers, log=TestFunctionRequests.LOG)
        print ("resp {}".format(resp))
        self.assertTrue(resp['args'].get("title"))
        self.assertEqual(resp['args'].get("title"), 'patch')

        # D E L E T E
        DEL_URL = "/".join((URL, "delete"))
        resp = rc.execute_call("delete", DEL_URL, None, log=TestFunctionRequests.LOG)
        self.assertEqual(resp.get("url"), DEL_URL)

        # bad verb
        with self.assertRaises(IntegrationError):
            resp = rc.execute_call("bad", URL, None, log=TestFunctionRequests.LOG)

    def test_statuscode(self):
        URL = TestFunctionRequests.URL_TEST_HTTP_STATUS_CODES

        rc = RequestsCommon(None, None)

        resp = rc.execute_call("get", "/".join((URL, "200")), None, resp_type='text')

        with self.assertRaises(IntegrationError):
            resp = rc.execute_call("get", "/".join((URL, "300")), None, resp_type='text')

    def test_statuscode(self):
        URL = "/".join((TestFunctionRequests.URL_TEST_HTTP_STATUS_CODES, "300"))

        def callback(resp):
            if resp.status_code != 300:
                raise ValueError(resp.status_code)

        rc = RequestsCommon(None, None)

        resp = rc.execute_call("get", URL, None, resp_type='text', callback=callback)

    def test_timeout(self):
        URL = "/".join((TestFunctionRequests.URL_TEST_HTTP_STATUS_CODES, "200?sleep=30000"))

        rc = RequestsCommon(None, None)

        with self.assertRaises(IntegrationError):
            resp = rc.execute_call("get", URL, None, resp_type='text', timeout=10)


    def test_basicauth(self):
        URL = "/".join((TestFunctionRequests.URL_TEST_HTTP_VERBS, "basic-auth"))
        basicauth = ("postman", "password")

        rc = RequestsCommon(None, None)

        resp = rc.execute_call("get", URL, None, basicauth=basicauth)
        self.assertTrue(resp.get("authenticated"))


    def test_proxy_override(self):
        rc = RequestsCommon(None, None)
        proxies = rc.get_proxies()
        self.assertIsNone(proxies)

        # test only integration proxies
        integrations_xyz =  {
            "integrations": {
                'http_proxy': "http://xyz.com",
                'https_proxy': "https://xyz.com"
            }
        }

        function_proxy_none =  {
            'http_proxy': None,
            'https_proxy': None
        }

        rc = RequestsCommon(integrations_xyz)
        proxies = rc.get_proxies()
        self.assertEqual(proxies['http'], "http://xyz.com")
        self.assertEqual(proxies['https'], "https://xyz.com")

        rc = RequestsCommon(integrations_xyz, function_proxy_none)
        proxies = rc.get_proxies()
        self.assertEqual(proxies['http'], "http://xyz.com")
        self.assertEqual(proxies['https'], "https://xyz.com")

        # test only function proxies
        integrations_none =  {
            "integrations": {
                'http_proxy': None,
                'https_proxy': None
            }
        }

        function_proxy_abc =  {
            'http_proxy': "http://abc.com",
            'https_proxy': "https://abc.com"
        }

        rc = RequestsCommon(function_opts=function_proxy_abc)
        proxies = rc.get_proxies()
        self.assertEqual(proxies['http'], "http://abc.com")
        self.assertEqual(proxies['https'], "https://abc.com")

        rc = RequestsCommon(integrations_none, function_proxy_abc)
        proxies = rc.get_proxies()
        self.assertEqual(proxies['http'], "http://abc.com")
        self.assertEqual(proxies['https'], "https://abc.com")


        # test integration and function proxies (override)
        rc = RequestsCommon(integrations_xyz, function_proxy_abc)
        proxies = rc.get_proxies()
        self.assertEqual(proxies['http'], "http://abc.com")
        self.assertEqual(proxies['https'], "https://abc.com")


    #@pytest.mark.skip(reason="may be over the limit")
    def test_proxy(self):
        rc = RequestsCommon()

        proxy_url = TestFunctionRequests.URL_TEST_PROXY
        proxy_result = rc.execute_call("get", proxy_url, None)

        proxies = {
            'http': proxy_result['curl'] if proxy_result['protocol'] == 'http' else None,
            'https': proxy_result['curl'] if proxy_result['protocol'] == 'https' else None
        }

        URL = "?".join((TestFunctionRequests.URL_TEST_DATA_RESULTS, "format=json"))

        # J S O N
        json_result = rc.execute_call("get", URL, None, proxies=proxies)

        self.assertTrue(json_result.get("ip"))

        integrations =  { "integrations": {
            'http_proxy': proxy_result['curl'] if proxy_result['protocol'] == 'http' else None,
            'https_proxy': proxy_result['curl'] if proxy_result['protocol'] == 'https' else None
        }
        }

        rc = RequestsCommon(opts=integrations)
        json_result = rc.execute_call("get", URL, None)
        self.assertTrue(json_result.get("ip"))


    def test_headers(self):
        # G E T with headers
        headers = {
            "Content-type": "application/json; charset=UTF-8",
            "my-sample-header": "my header"
        }
        URL = "/".join((TestFunctionRequests.URL_TEST_HTTP_VERBS, "headers"))

        rc = RequestsCommon()

        json_result = rc.execute_call("get", URL, None, headers=headers)
        self.assertEqual(json_result['headers'].get("my-sample-header"), "my header")


    @parameterized.expand([
        [None, True],
        ["application/json", True],
        ["application/json; charset=UTF-8", True],
        ["charset=UTF-8; application/json", True],
        ["application/x-www-form-urlencoded", False]
    ])
    def test_is_payload_in_json(self, content_type, result):

        payload_in_json = is_payload_in_json(content_type)

        self.assertEqual(payload_in_json, result)


    @parameterized.expand([
        [None, None],
        [{"Content-Type": "mock_type"}, "mock_type"],
        [{"my-sample-header": "my header"}, None]
    ])
    def test_get_case_insensitive_key_value(self, dictionary, result):

        value = get_case_insensitive_key_value(dictionary, "content-type")

        self.assertEqual(value, result)



# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2018. All Rights Reserved.
# pragma pylint: disable=unused-argument, no-self-use

import re
import logging
from six import string_types

try:
    from HTMLParser import HTMLParser
except:
    from html.parser import HTMLParser

class MarkdownParser(HTMLParser):
    QUILL_RTE = "rte"       # first <div> will have this class. This is part of quill

    HTML_STYLE_COLOR = r'rgb\(([\d]+),[\s]*([\d]+),[\s]*([\d]+)\)'
    SUPPORTED_TAGS = ["div", "span", "br", "strong", "em", "s", "u", "ol", "ul", "li", "a",
                      "h", "h1", "h2", "h3", "h4", "h5", "h6", "blockquote"]

    MARKDOWN_NEWLINE = "\n"
    MARKDOWN_NEWSECTION = "\n\n"
    
    DEFAULT_LIST = "*"

    def __init__(self, bold="**", italic="*", underline="__", strikeout="~~", bullets=DEFAULT_LIST, number=0, indent=4,
                 monospace=["{{", "}}"], headers=['h1.', 'h2.', 'h3.', 'h4.', 'h5.', 'h6.'], blockquote="```"):
        HTMLParser.__init__(self)
        self.log = logging.getLogger(__name__)

        # customizable attributes
        self.bold = bold
        self.italic = italic
        self.underscore = underline
        self.strikeout = strikeout
        self.list_bullets = bullets if isinstance(bullets, list) else [bullets] * 6
        self.list_number = number
        self.indent = indent
        self.monospace = monospace
        self.headers = headers
        self.blockquote = blockquote

    def init_buffers(self):
        self.buffer = []      # end markdown buffer
        self.curr_tag = []    # stack of tags to track
        self.curr_attrs = []  # stack of tag attributes to track
        self.curr_list = []   # stack of embedded ordered and unordered list symbols
        self.data = []        # buffer for a given tag, cleared when and ending tag is found (ex. </p>)
        self.data_pre = []    # markdown data to prefix the data
        self.data_post = []   # markdown data to follow the data
        self.prev_tag = None
        self.prev_attrs = []

    def convert(self, data):
        """
        starting point for app, wrapper to htmlparser.feed
        :param data: html string
        :return: converted text to markdown
        """

        self.init_buffers()

        if not data or not isinstance(data, string_types):
            return data

        self.feed(data)
        return self.toString()

    def handle_starttag(self, tag, attrs):
        """
        handler for the start of tags. Logic is added to surround data with markdown
        :param tag:
        :param attrs:
        :return: None
        """

        # flush any data accumulated
        if tag in ('ol', 'ul'):
            self.push_data(False)
        else:
            self.push_data(True)

        # retain the hierarchy of nested command, which may be needed
        self.curr_tag.append(tag)
        self.curr_attrs.append(attrs)

        if tag == "div":
            if self.prev_tag in ("ol", "ul"):
                self.data_pre.append(MarkdownParser.MARKDOWN_NEWSECTION)

        if tag == "strong":
            self.data_pre.append(self.bold)
            self.data_post.insert(0, self.bold)

        elif tag == "em":
            self.data_pre.append(self.italic)
            self.data_post.insert(0, self.italic)

        elif tag == "s":
            self.data_pre.append(self.strikeout)
            self.data_post.insert(0, self.strikeout)

        elif tag == "u":
            self.data_pre.append(self.underscore)
            self.data_post.insert(0, self.underscore)

        elif tag == "ol":
            self.curr_list.append(self.list_number)  # number to be incremented with every <li>

        elif tag == "ul":
            if self.list_bullets:
                self.curr_list.append(self.list_bullets.pop(0))  # get the symbol to use
            else:
                # default list marker
                self.curr_list.append(MarkdownParser.DEFAULT_LIST)

        elif tag == "li":
            # add proper # of spaces
            self.data_pre.append(MarkdownParser.MARKDOWN_NEWLINE)
            self.data_pre.append((" " * self.indent) * len(self.curr_list))

            if not isinstance(self.curr_list[-1], int):
                self.data_pre.append('{} '.format(self.curr_list[-1]))
            else:
                num = self.curr_list.pop()
                num = num+1
                self.curr_list.append(num)
                self.data_pre.append("{}. ".format(num))

        elif tag == "a":
            href = self.get_attr(attrs, 'href')
            self.data_pre.extend(["[{}]".format(href), '('])
            self.data_post.insert(0, ")")

        elif tag in ('h', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6'):
            if tag == 'h':
                tag = 'h1'
            # get the number of header and use as index into the header array
            idx = int(tag[-1:])-1
            if idx < len(self.headers):
                self.data_pre.append("{} ".format(self.headers[idx]))

            self.data_post.append(MarkdownParser.MARKDOWN_NEWLINE)

        elif tag == "blockquote":
            self.data_pre.append(self.blockquote)
            self.data_post.insert(0, self.blockquote)

        elif tag not in MarkdownParser.SUPPORTED_TAGS:
            self.log.warning("Unknown html tag: {}".format(tag))
            self.data_post.insert(0, MarkdownParser.MARKDOWN_NEWLINE)

        # determine if styling is needed
        style = self.get_attr(attrs, 'style')
        if style:
            rgb = self.get_style_attr(style, 'color')
            if rgb:
                rgb_hex = self.convert_rgb(self.get_rgb(rgb))
                self.data_pre.append("{{color:{0}}}".format(rgb_hex))
                self.data_post.insert(0, "{color}")

            # format monospace data blocks
            font_family = self.get_style_attr(style, 'font-family')
            if font_family and font_family == "monospace":
                if isinstance(self.monospace, list):
                    self.data_pre.append(self.monospace[0])
                    self.data_post.insert(0, self.monospace[1])
                else:
                    self.data_pre.append(self.monospace)
                    self.data_post.insert(0, self.monospace)


    def handle_data(self, data):
        """
        data handler. Basically, add tagged data to the growing buffer
        :param data:
        :return: None
        """
        # clean data of prefix whitespace
        cleaned = re.search(r"^[\n\t\r]*(.*)", data, re.S)
        cleaned and self.data.append(cleaned.group(1))


    def handle_endtag(self, tag):
        """
        handler for end tags.
        :param tag:
        :return: None
        """

        # remove existing tag from stack
        self.prev_tag = self.curr_tag.pop()
        self.prev_attrs = self.curr_attrs.pop()

        if self.prev_tag != tag:
            raise ValueError("Mismatch tag {} expecting {}".format(tag, self.prev_tag))

        if tag == "div":
            self.data_post.append(MarkdownParser.MARKDOWN_NEWLINE)

        elif tag in ("ol", "ul"):
            if len(self.curr_list) > 0:
                bullet = self.curr_list.pop()   # remove existing bullet, exposing previous bullet
                if tag == "ul":
                    if self.list_bullets:
                        self.list_bullets.insert(0, bullet)      # clear top item on list symbols
                    else:
                        self.list_bullets = [bullet]

        elif tag == "br":
            # this is data rather than pre or post data
            self.data.append(MarkdownParser.MARKDOWN_NEWSECTION)

        self.push_data()


    def push_data(self, data_only=False):
        """
        flush the data buffer and reset for next html tag
        :return: None
        """

        if data_only:
            self.buffer.extend(self.data)
            self.data = []
        else:
            self.buffer.extend([item for sublist in [self.data_pre, self.data, self.data_post] for item in sublist])

            # clean up
            self.data = []
            self.data_pre = []
            self.data_post = []


    def convert_rgb(self, rgb):
        """
        convert rgb values to hexcode format
        :param rgb:
        :return:
        """
        return '#'+''.join('%02x'% int(i) for i in rgb)


    def get_attr(self, attrs, key):
        """
        get an attribute from the data's previous block
        ex. <div style='font-family: monospace'>zzz</div>
        :param attrs:
        :param key:
        :return: found attribute or None
        """
        for attr in attrs:
            if attr[0] == key:
                return attr[1]

        return None


    def get_style_attr(self, style, key):
        """
        find css data within the style attribute
        :param style:
        :param key: css label
        :return: css data
        """
        for attr in style.split(';'):
            attr_split = attr.split(':')
            if attr_split[0].strip() == key:
                return attr_split[1].strip()

        return None


    def get_rgb(self, str):
        """
        format of rgb information is "rgb(rrr, ggg, bbb)"
        :param str:
        :return: list of values
        """
        m = re.search(MarkdownParser.HTML_STYLE_COLOR,  str)
        return m.group(1, 2, 3) if m else None


    def __str__(self):
        return self.toString()

    def __repr__(self):
        return self.toString()

    def toString(self):
        """
        we're done. flush the data buffer and print entire content
        :return: markdown result
        """
        self.push_data()

        result = ''.join(self.buffer)
        # clean up ending new line characters
        while result[-1:] == '\n':
            result = result[:-1]

        return result



# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2019. All Rights Reserved.
# pragma pylint: disable=unused-argument, no-self-use


class WorkflowStatus(object):
    """Class to handle the status of a Workflow Instance"""
    def __init__(self, instance_id, status, start_date, end_date=None, reason=None):
        self.instance_id = instance_id
        self.status = status
        self.start_date = start_date
        self.end_date = end_date
        self.reason = reason
        self.is_terminated = True if status == "terminated" else False

    def __str__(self):
        return "WorkflowStatus {0}: {1}".format(self.instance_id, self.as_dict())

    def as_dict(self):
        """Returns this class object as a dictionary"""
        return self.__dict__

def get_workflow_status(res_client, wf_instance_id):
    """Function to check if the current workflow has been terminated via the UI
    :param res_client: reference to Resilient rest_client()
    :param wf_instance_id: the workflow_instance_id of the current Workflow
    :rtype: WorkflowStatus Object
    """

    if isinstance(wf_instance_id, int) is False:
        raise ValueError("wf_instance_id must be of type int and not {0}".format(type(wf_instance_id)))

    resp = res_client.get("/workflow_instances/{0}".format(wf_instance_id))

    return WorkflowStatus(
        instance_id=wf_instance_id,
        status=resp.get("status"),
        start_date=resp.get("start_date"),
        end_date=resp.get("end_date"),
        reason=resp.get("terminate_reason"))



#!/usr/bin/env python
__import__('pkg_resources').declare_namespace(__name__)

from resilient_lib.components.function_result import ResultPayload
from resilient_lib.components.html2markdown import MarkdownParser
from resilient_lib.components.requests_common import RequestsCommon
from resilient_lib.components.resilient_common import *
from resilient_lib.components.workflow_status import get_workflow_status



import unittest
import json
from resilient_lib.components.function_result import ResultPayload

class TestFunctionMetrics(unittest.TestCase):
    """ Tests for the attachment_hash function"""

    def test_function_result(self):
        pgkname = "requests"
        params = { "param1": "value1",
                    "param2": "value2",
                    "param3": "value3"
                 }
        fr = ResultPayload(pgkname, **params)

        result = { "result1": "value1",
                   "result2": "value2",
                   "result3": "value3"
                 }
        result_dumps = json.dumps(result)

        result = fr.done(True, result)

        self.assertIsNotNone(result)
        self.assertIsNotNone(result['success'])
        self.assertTrue(result['success'])
        self.assertEqual(result['reason'], None)
        self.assertEqual(result['content']["result1"], "value1")
        self.assertEqual(result['raw'], result_dumps)
        self.assertIsNotNone(result.get('metrics'))
        self.assertEqual(result.get('metrics')['package'], pgkname)
        self.assertIsNotNone(result.get('inputs'))
        self.assertEqual(result['inputs']['param1'], 'value1')
